# build_parameters.R
#
#

# ------------------------------------------------------------------------------
# gather command line arguments
# ------------------------------------------------------------------------------

cmd.arg = commandArgs(trailingOnly=TRUE)



# ------------------------------------------------------------------------------
# set general parameters
# ------------------------------------------------------------------------------

# relative path to where the data files are stored
data.dir = file.path("build","data")

# save the years under a new name for use in this file
model.years = sage.years

# name of file to write the elasticities and years to include in the model
if (cmd.arg[1] == "static") {
    output.file = file.path("data","parameters_static.gms")
    model.years = model.years[1]
} else if (cmd.arg[1] == "balanced") {
    output.file = file.path("data","parameters_balanced_growth.gms")
} else {
    output.file = file.path("data","parameters.gms")
}



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

library(httr)
library(RJSONIO)

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))



# ------------------------------------------------------------------------------
# armington elasticities
# ------------------------------------------------------------------------------

# load the gtap elasticities
file.name = "hertel et al - GTAP v7 - behavioral parameters - table 14-2.csv"
gtap = read.csv(file.path(data.dir,file.name),
                stringsAsFactors=FALSE,header=TRUE,skip=19)

# load the gtap to sage mapping
gtap.map = read.csv(file.path(data.dir,"gtap_to_sage.csv"),
                    stringsAsFactors=FALSE,header=TRUE,skip=1)

# get the list of sage sectors
sage.sectors = unique(gtap.map$sage)

# for each sage sector get the armington elasticities
se_nf = NULL
se_dn = NULL
for (s in sage.sectors) {

  # gtap sectors within the current sage sector
  ind = gtap$GTAP.Sector %in% gtap.map$gtap[gtap.map$sage==s]

  # compute the value weighted national-foreign armington elasticity
  # weighting is by import value at world prices as reported in GTAP v9
  se_nf_value = sum(gtap$sigma_D[ind]*gtap$viws[ind])/sum(gtap$viws[ind])

  # crude oil and natural gas elasticities from hertel et al. (2007) are less
  # appropriate for the U.S. and therefore, we follow caron and rausch (2013)
  # in there updating of the hertel et al. (2007) elasticities for these sectors
  if (s=="cru")
    se_nf_value = 7.3
  if (s=="gas")
    se_nf_value = 2.8

  se_nf = rbind(se_nf,cbind(s,se_nf_value))

  # coughlin and novy (2013) ratio of intra-national to international border
  # effects
  delta = 1.868

  # caron and rausch (2013) relationship between domestic-national and national-
  # foreign armington elasticities
  se_dn = rbind(se_dn,cbind(s,.5-delta*(.5-se_nf_value)))

}



# ------------------------------------------------------------------------------
# transformation elasticity of output
# ------------------------------------------------------------------------------

# domestic vs. export transformation elasticity of output
# taken from us-rep
te_dx = 2



# ------------------------------------------------------------------------------
# transformation elasticity for sector differentiated extant capital
# ------------------------------------------------------------------------------

# transformation elasticity for sector differentiated extant capital
te_k_ex = 1.5



# ------------------------------------------------------------------------------
# klem substitution elasticities
# ------------------------------------------------------------------------------

# load the koesler and schymura (2015) elasticities
klem = read.csv(file.path(data.dir,
                          "koesler and schymura - 2015 - klem elasticities.csv"),
                stringsAsFactors=FALSE,header=TRUE,skip=8)

# for sectors without kl elasticities (either due to inf answers or
# convergencece failure na) and without other sectors being mapped to the sage
# sector use the young (2013) gmm non-normalized values instead
klem$se_kl[klem$nace=="23"] = 0.732      # petroleum refining
klem$se_kl[klem$nace=="E"]  = 1.002      # water and electric utilities

# petroleum refining has an inf value for the kle elasticity, instead use the
# the total industries value
klem$se_kle[klem$nace=="23"] = klem$se_kle[klem$nace=="TOT"]

# load the nace code to sage mapping
nace.map = read.csv(file.path(data.dir,"nace_to_sage.csv"),
                    stringsAsFactors=FALSE,header=TRUE)

# for each sage sector get the value added elasticity
se_kl = NULL
se_kle = NULL
se_klem = NULL
for (s in sage.sectors) {

  # nace sectors within the current sage sector
  ind = klem[,1] %in% nace.map$nace[nace.map$sage==s]

  # compute the value weighted value added elasticity
  # weighting is by output in the last year of the koesler and schymura sample
  # only include non-NA and non-Inf values
  ind_kl = ind & !(is.na(klem$se_kl) | klem$se_kl=="Inf")
  y = sum(as.double(klem$output[ind_kl]))
  kl_value = sum(as.double(klem$se_kl[ind_kl])*as.double(klem$output[ind_kl]))/y

  # compute the remaining klem elasticities
  # weighting is by output in the last year of the koesler and schymura sample
  y = sum(as.double(klem$output[ind]))
  kle_value = sum(as.double(klem$se_kle[ind])*as.double(klem$output[ind]))/y
  klem_value = sum(as.double(klem$se_klem[ind])*as.double(klem$output[ind]))/y

  # write the klem elasticities to the output file
  se_kl = rbind(se_kl,cbind(s,kl_value))
  se_kle = rbind(se_kle,cbind(s,kle_value))
  se_klem = rbind(se_klem,cbind(s,klem_value))

}



# ------------------------------------------------------------------------------
# interfuel energy substitution elasticities
# ------------------------------------------------------------------------------

# mapping between energy commodities and seds energy commodity codes
fuel.map = data.frame(fuel=c("col","ref","gas","ele"),
                      seds=c("CL","PA","NG","ES"),
                      stringsAsFactors=FALSE)

# mapping between aggregate sector names and seds codes
sector.map = data.frame(sector=c("industrial","commercial","electricity",
                                 "transportation","residential"),
                        seds=c("I","C","E","A","R"),
                        stringsAsFactors=FALSE)

# create a container for the consumption data
en.consumption = expand.grid(sector.map$sector,fuel.map$fuel,
                             stringsAsFactors=FALSE)
names(en.consumption) = c("sector","fuel")

# get the seds national level consumption in btus of each energy commodity by
# aggregate sector in the model's benchmark year
en.consumption$value = 0
for (i in 1:nrow(en.consumption)) {
  if (en.consumption$sector[i]=="electricity" & en.consumption$fuel[i]=="ele")
    next
  if (en.consumption$sector[i]=="electricity")
    c.var = "IB"
  else
    c.var = "CB"
  en.consumption$value[i] =
    get.eia(paste0("SEDS.",
                   fuel.map$seds[fuel.map$fuel==en.consumption$fuel[i]],
                   sector.map$seds[sector.map$sector==en.consumption$sector[i]],
                   c.var,".US.A"),year=model.years[1])$value
}

# get the seds national level price of each energy commodity by aggregate sector
# in the model's benchmark year
en.consumption$price = 0
for (i in 1:nrow(en.consumption)) {
  if (en.consumption$sector[i]=="electricity" & en.consumption$fuel[i]=="ele")
    next
  if (en.consumption$sector[i]=="electricity")
    p.var = "ID"
  else
    p.var = "CD"
  en.consumption$price[i] =
    get.eia(paste0("SEDS.",
                   fuel.map$seds[fuel.map$fuel==en.consumption$fuel[i]],
                   sector.map$seds[sector.map$sector==en.consumption$sector[i]],
                   p.var,".US.A"),year=model.years[1])$value
}

# consumption cost by aggregate sector and fuel
en.consumption$cost = en.consumption$value*en.consumption$price

# create a container for the aggregate sector elasticities between primary
# energy sources
se_en_agg = data.frame(sector=unique(sector.map$sector),stringsAsFactors=FALSE)

# serletis et al. (2010) allen substitution elasticities between refined
# petroleum and natural gas for the industril sector from table 7
se_en_agg$value[se_en_agg$sector=="industrial"] = 0.331

# serletis et al. (2010) results for the residential and commerical sector
# suggest very little substitution possibility between refined petorleum and
# natural gas so the substitution elasticities are set commensurately
se_en_agg$value[se_en_agg$sector=="residential"] = 0.1
se_en_agg$value[se_en_agg$sector=="commercial"] = 0.1

# serletis et al. (2010) allen substitution elasticities between coal and
# natural gas for the electricity sector from table 10
se_en_agg$value[se_en_agg$sector=="electricity"] = 0.227

# transportation based on serletis et al (2010b) international study
se_en_agg$value[se_en_agg$sector=="transportation"] = 0.25

# create a container for the aggregate sector elasticities between primary
# energy and electricity
se_ene_agg = data.frame(sector=unique(sector.map$sector),stringsAsFactors=FALSE)

# serletis et al. (2010) allen substitution elasticities between refined
# petroleum and electricity and natural gas and electricity from tables 7-9
sigma_ref_ele = c(0.24,1.532,1.280)
names(sigma_ref_ele) = c("industrial","residential","commercial")
sigma_gas_ele = c(1.693,0.675,0.497)
names(sigma_gas_ele) = c("industrial","residential","commercial")

# variable to use when weighting the elasticities
weight.variable = "cost"

# for the industrial, residential, and commercial sectors set the elasticity to
# a weighted average of the gas to electricity and refined petroleum to
# electricity elasticities
for (s in c("industrial","residential","commercial")) {

  ind = en.consumption$sector==s

  se_ene_agg$value[se_ene_agg$sector==s] =
    (en.consumption[ind & en.consumption$fuel=="ref",weight.variable]*
       sigma_ref_ele[s]+
     en.consumption[ind & en.consumption$fuel=="gas",weight.variable]*
       sigma_gas_ele[s])/
    (en.consumption[ind & en.consumption$fuel=="ref",weight.variable]+
     en.consumption[ind & en.consumption$fuel=="gas",weight.variable])

}

# virtually eliminate substitution possibilities between electricity and primary
# energy inputs in the electricity sector
se_ene_agg$value[se_ene_agg$sector=="electricity"] = 0.01

# transportation based on serletis et al (2010b) international study
se_ene_agg$value[se_ene_agg$sector=="transportation"] = 0.25

# commercial elasticities apply to the service sectors
com = c("hlt","srv")

# transportation sectors
trn = c("trn","ttn")

# for each sage sector get the energy elasticities
se_en = NULL
se_ene = NULL
for (s in sage.sectors) {

  # use the commercial elasticity index for designated commercial sectors
  if (s %in% com)
    sector = "commercial"
  else if (s == "ele")
    sector = "electricity"
  else if (s %in% trn)
    sector = "transportation"
  else
    sector = "industrial"

  # save sector specific elasticities
  se_en = rbind(se_en,cbind(s,se_en_agg$value[se_en_agg$sector==sector]))
  se_ene = rbind(se_ene,cbind(s,se_ene_agg$value[se_ene_agg$sector==sector]))

}

# save the energy elasticities for households
se_cen = se_en_agg$value[se_en_agg$sector=="residential"]
se_cene = se_ene_agg$value[se_ene_agg$sector=="residential"]



# ------------------------------------------------------------------------------
# labor supply elasticities
# ------------------------------------------------------------------------------

# labor supple elasticity - substitution effect
# based on most recent cbo literature review midpoints
lse_sub = 0.20

# labor supple elasticity - income effect
# based on most recent cbo literature review midpoints
lse_inc = -0.05



# ------------------------------------------------------------------------------
# household consumption elasticities
# ------------------------------------------------------------------------------



# substitution elasticity across non-energy and energy goods
# from usrep
se_c = 0.25;

# substituion elasticity across non-energy goods
# from usrep
se_cm = 0.25;

# inverse elasticity of substitution of consumption over time
eta = 1.66;



# ------------------------------------------------------------------------------
# fixed factor shares in resource and agriculture sectors
# ------------------------------------------------------------------------------

# fixed factor resource share of capital for resource extraction and
# agrictultural and forestry sectors
res_share = cbind(c("col","cru","gas","min","agf"),c(0.40,.25,0.25,0.40,0.40))



# ------------------------------------------------------------------------------
# resource extraction and agricutlure sector supply elasticities
# ------------------------------------------------------------------------------

# long-run supply elasticity for resoursce extraction and agrictultural and
# forestry sectors
se_res_lr = cbind(c("col","cru","gas","min","agf"),c(2.4,0.15,0.5,5,0.4))



# ------------------------------------------------------------------------------
# macroeconomic variables
# ------------------------------------------------------------------------------

# define versioning matrix
data.versions = NULL

# benchmark population
# Macroeconomic Indicators : Key Labor Indicators : Labor Force, Reference, AEO
id = paste0("AEO.",aeo.year,".",aeo.scenario,".ECI_LBF_NA_NA_NA_NA_NA_MILL.A")
n = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

# rate - population growth rate
# based on average labor force growth rate in AEO
gamma = mean(n$value[-nrow(n)]/n$value[-1]-1)

# rate - labor productivity growth
# Macroeconomic Indicators : Key Labor Indicators : Nonfarm Labor Productivity
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".ECI_LP_NOFM_NA_NA_NA_NA_Y09EQ1.A")
temp = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

omega = mean(temp$value[-nrow(temp)]/temp$value[-1]-1)

# rate - depreciation rate
delta = 0.05

# rate - benchmark interest rate
rbar = 0.045

# if this is a static model run then redefine the rate parameters - in the
# static run these parameters essentially lose their meaning so these values are
# just to ensure the solution represents a static run and the values should not
# be interpreted with the parameters normal meaning for the dynamic model
if (length(model.years)==1) {
  gamma = 0.0
  delta = 1.0
  omega = 0.0
  rbar = 0.0
}



# ------------------------------------------------------------------------------
# average energy intensity growth rate
# ------------------------------------------------------------------------------

# baseline improvements in the energy intensity of production are based on the
# unit energy consumption in eia's aeo forecast. for each sage sector we compute
# the gorwth rate of consumption per unit of real output aggregated across the
# nems sectors within that sage sector over the aeo time horizon. the mapping of
# nems to sage sectors is located in build/data/nems_to_sage.csv

# initialize space for the value of shipements and energy consumption data
output = NULL
energy = NULL

ids = read.csv(file.path("build","data","aeo_api_ids_by_sector.csv"),
               stringsAsFactors=FALSE)

# for each nems sector retrieve the data using the eia api
for (i in 1:nrow(ids)) {

  # add the output data for the sector to the dataset
  id = paste0("AEO.",aeo.year,".",aeo.scenario,".",ids$y[i],".A")
  temp = get.eia(id)
  data.versions = rbind(data.versions,get.eia.version(id))
  temp$sector = ids$nems[i]
  output = rbind(output,temp)

  # add the energy consumption data for the sector to the dataset
  for (j in 3:ncol(ids)) {
    if (is.na(ids[i,j]))
      next
    id = paste0("AEO.",aeo.year,".",aeo.scenario,".",ids[i,j],".A")
    temp = get.eia(id)
    data.versions = rbind(data.versions,get.eia.version(id))
    temp$sector = ids$nems[i]
    temp$source = names(ids)[j]
    energy = rbind(energy,temp)
  }

}

# load the nems to sage mapping
map = read.csv(file.path(data.dir,"nems_to_sage.csv"),
               stringsAsFactors=FALSE,header=TRUE)

# get the list of sage sectors
sage.sectors = unique(map$sage)

# for each sage sector get the energy intensities
ene_growth = NULL
for (s in sage.sectors) {

  if (any(map$nems[map$sage==s]==""))
    next

  # get energy and output for the nems sectors in the sage sector
  sector.energy = energy[energy$sector %in% map$nems[map$sage==s],]
  sector.output = output[output$sector %in% map$nems[map$sage==s],]

  # aggregate across nems sectors by year
  sector.energy = aggregate(sector.energy$value,by=list(sector.energy$year),sum)
  sector.output = aggregate(sector.output$value,by=list(sector.output$year),sum)

  names(sector.energy) = c("year","value")
  names(sector.output) = c("year","value")

  # average energy intensity growth rate
  intensity = sector.energy
  intensity$value = sector.energy$value/sector.output$value
  growth = (intensity$value[nrow(intensity)]/
            intensity$value[1])^(1/(nrow(intensity)-1))-1
  ene_growth = rbind(ene_growth,data.frame(ss=c("gas","ele","col","ref"),s=s,
                                           value=growth))

}

ene_growth = ene_growth[!is.nan(ene_growth$value),]

# energy intensity growth for the transportation sector is based on fuel
# efficiency improvements forecast for airline travel as reported in aeo
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_NA_TRN_AIR_USE_NA_NA_QBTU.A")
temp = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".KEI_TRV_TRN_NA_AIR_NA_NA_BLNSEATMLS.A")
temp$value = temp$value/get.eia(id)$value
data.versions = rbind(data.versions,get.eia.version(id))

# set the growth rate
# the eia api returns the last year on top so temp$value[1] is the last year
growth_rate = (temp$value[1]/temp$value[nrow(temp)])^(1/(nrow(temp)-1))-1
ene_growth = rbind(ene_growth,data.frame(ss=c("ref"),s="trn",value=growth_rate))

# energy intensity growth for the truck transportation sector is based on fuel
# efficiency improvements forecast for frieght truck travel as reported in aeo
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_NA_TRN_FGHT_USE_NA_NA_QBTU.A")
temp = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".KEI_TRV_TRN_NA_FGHT_NA_NA_BLNVEHMLS.A")
temp$value = temp$value/(get.eia(id)$value)
data.versions = rbind(data.versions,get.eia.version(id))

# set the growth rate
# the eia api returns the last year on top so temp$value[1] is the last year
growth_rate = (temp$value[1]/temp$value[nrow(temp)])^(1/(nrow(temp)-1))-1
ene_growth = rbind(ene_growth,data.frame(ss=c("ref"),s="ttn",value=growth_rate))



# ------------------------------------------------------------------------------
# average growth rate of coal fired electricity generation share
# ------------------------------------------------------------------------------

# electricity output valued at initial year prices
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".GEN_NA_ELEP_CHP_TTG_NA_USA_BLNKWH.A")
ele = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

# coal input valued at initial year prices
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_ENU_ELEP_NA_STC_NA_NA_QBTU.A")
#id = paste0("AEO.",aeo.year,".",aeo.scenario,
#            ".CNSM_NA_ELEP_NA_CL_NA_NA_MILLTON.A")
col = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

# coal fired electricity generation share
share = col$value/ele$value

# average growth rate of coal fired electricity generation share
col_ele_growth = (share[1]/share[length(share)])^(1/(length(share)-1))-1



# ------------------------------------------------------------------------------
# household energy intensity growth
# ------------------------------------------------------------------------------

# consumption forecast in aeo
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".ECI_NA_NA_NA_GDP_RCNSP_NA_BLNY09DLR.A")
c = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

# consumption electricity intensity growth is based on residential electricity
# consumption growth in aeo relative to consumption growth
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_NA_RESD_NA_ELS_NA_USA_BLNKWH.A")
e = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

# set the growth rate
# the eia api returns the last year on top so temp$value[1] is the last year
c.share = e$value/c$value
growth = (c.share[1]/c.share[length(c.share)])^(1/(length(c.share)-1))-1
cd_ene_growth = data.frame(s="ele",value=growth)

# consumption natural gas intensity growth is based on residential natural gas
# consumption growth in aeo relative to consumption growth
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_ENU_RESD_NA_NG_NA_NA_QBTU.A")
e = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

# set the growth rate
# the eia api returns the last year on top so temp$value[1] is the last year
c.share = e$value/c$value
growth = (c.share[1]/c.share[length(c.share)])^(1/(length(c.share)-1))-1
cd_ene_growth = rbind(cd_ene_growth,data.frame(s="gas",value=growth))

# consumption refined oil intensity growth is based on light duty vehicle energy
# intensity growth in aeo
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_NA_TRN_LDV_USE_NA_NA_QBTU.A")
e = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

# implicitly assumes that households share of overall light duty vehicle miles
# traveled will remain constant
c.share = e$value/c$value
growth = (c.share[1]/c.share[length(c.share)])^(1/(length(c.share)-1))-1
cd_ene_growth = rbind(cd_ene_growth,data.frame(s="ref",value=growth))

# zero out any change in coal
cd_ene_growth = rbind(cd_ene_growth,data.frame(s="col",value=0))



# ------------------------------------------------------------------------------
# write parameters to the input file
# ------------------------------------------------------------------------------

# open a connection to the output file
fc = file(output.file,"w")

# write the set defining the model's years
writeLines("sets",con=fc)
t = model.years[1]
if (length(model.years)>1)
  for (i in 2:length(model.years))
    t = paste0(t,", ",model.years[i])
writeLines(paste0("  t                     time periods            / ",t," /;"),
           con=fc)
writeLines("",con=fc)

# armington elasticities
comments = "substitution elasticity - national-foreign armington elasticity"
write.gams.variable(fc,"se_nf",se_nf,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - domestic-national armington elasticity"
write.gams.variable(fc,"se_dn",se_dn,comments=comments,write.zeros=TRUE)

# transformation elasticity - domestic and exported production
comments = "transformation elasticity - domestic and exported production"
write.gams.variable(fc,"te_dx(s)",te_dx,comments=comments,write.zeros=TRUE)

# transformation elasticity for sector differentiated extant capital
comments = "transformation elasticity for sector differentiated extant capital"
write.gams.variable(fc,"te_k_ex",te_k_ex,comments=comments,write.zeros=TRUE)

# klem substitution elasticities
comments = "substitution elasticity - value added"
write.gams.variable(fc,"se_kl",se_kl,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - value-added-energy bundle"
write.gams.variable(fc,"se_kle",se_kle,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - materials and value-added-energy bundle"
write.gams.variable(fc,"se_klem",se_klem,comments=comments,write.zeros=TRUE)

# energy substitution elasticities
comments = "substitution elasticity - primary energy bundle"
write.gams.variable(fc,"se_en",se_en,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - electricity and primary energy bundle"
write.gams.variable(fc,"se_ene",se_ene,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - consumption primary energy bundle"
write.gams.variable(fc,"se_cen",se_cen,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - consumption electricity and energy bundle"
write.gams.variable(fc,"se_cene",se_cene,comments=comments,write.zeros=TRUE)

# labor supply elasticities
comments = "labor supply elasticity - substitution"
write.gams.variable(fc,"lse_sub",lse_sub,comments=comments,write.zeros=TRUE)
comments = "labor supply elasticity - income"
write.gams.variable(fc,"lse_inc",lse_inc,comments=comments,write.zeros=TRUE)

# household consumption elasticities
comments = "substitution elasticity - consumption bundle"
write.gams.variable(fc,"se_c",se_c,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - consumption materials bundle"
write.gams.variable(fc,"se_cm",se_cm,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - inverse of consumption over time"
write.gams.variable(fc,"eta(r,h)",eta,comments=comments,write.zeros=TRUE)

if (cmd.arg[1] != "balanced") {

    # fixed factor shares in resource and agriculture sectors
    comments = "fixed factor resource share of capital"
    write.gams.variable(fc,"res_share",res_share,comments=comments,
                        write.zeros=TRUE)

    # resource extraction and agriculture and forestry supply elasticities
    comments = "long-run supply elasticity for sectors with fixed factor"
    write.gams.variable(fc,"se_res_lr",se_res_lr,comments=comments,
                        write.zeros=TRUE)
}

# macroeconomic variables
comments = "rate - population growth rate"
write.gams.variable(fc,"gamma",gamma,comments=comments,write.zeros=TRUE)
comments = "rate - labor productivity growth"
write.gams.variable(fc,"omega",omega,comments=comments,write.zeros=TRUE)
comments = "rate - depreciation rate"
write.gams.variable(fc,"delta",delta,comments=comments,write.zeros=TRUE)
comments = "rate - benchmark interest rate"
write.gams.variable(fc,"rbar",rbar,comments=comments,write.zeros=TRUE)

if (cmd.arg[1] == "dynamic") {

    # average energy intesity growth rate
    comments = "energy intesity growth rate"
    write.gams.variable(fc,"ene_growth",ene_growth,comments=comments,
                        write.zeros=TRUE)

    # average growth rate of coal fired electricity generation share
    comments = "growth rate of coal fired electricity generation share"
    write.gams.variable(fc,"col_ele_growth",col_ele_growth,comments=comments,
                        write.zeros=TRUE)

    # household energy intensity growth
    comments = "household energy intensity growth"
    write.gams.variable(fc,"cd_ene_growth",cd_ene_growth,comments=comments,
                        write.zeros=TRUE)
}

# close the connection to the output file
close(fc)



# ------------------------------------------------------------------------------
# save the r data image for versioning and generate report
# ------------------------------------------------------------------------------

# save data versioning eia parameter
data.versions = data.frame(data.versions)
colnames(data.versions)[1] = "agency"
colnames(data.versions)[2] = "call_id"
colnames(data.versions)[3] = "version"
write.csv(data.versions,
          file="build/data/satellite_data_versions.csv",
          row.names=FALSE)

# save data image for posterity
save.image(file=paste0("build/data/build_parameters_image.RData"))
