* simulates the uncompensated and compensated labor supply elasticities
*
* run as an "extra code file" using:
*   gams model/sage.gms --extra_code_file=examples/labor_supply_elasticity.gms
*
* see examples/labor_supply_elasticity.R for a run script with results plot
*



* ------------------------------------------------------------------------------
* options
* ------------------------------------------------------------------------------

* set default name for output file if needed
$if not set lse_output_file $set lse_output_file "output\lse_results.csv"



* ------------------------------------------------------------------------------
* save the baseline results
* ------------------------------------------------------------------------------

parameters
  l_base(t,r,h)         baseline labor supply
  pl_base(t,r)          baseline wage rate
  w_base(r,h)           baseline welfare;

* save the baseline variables
l_base(t,r,h) = l.l(t,r,h);
pl_base(t,r) = pl.l(t,r);
w_base(r,h) = w.l(r,h);



* ------------------------------------------------------------------------------
* define mcp problem for the uncompensated elasticitiy
* ------------------------------------------------------------------------------

* fix the prices at the policy case levels
pa.fx(t,r,s)      = pa.l(t,r,s);
pk.fx(t,r)        = pk.l(t,r);
pr.fx(t,r)        = pr.l(t,r);
pl.fx(t,r)        = pl.l(t,r);
pfx.fx(t)         = pfx.l(t);
pkt.fx(r)         = pkt.l(r);
kt.fx(r)          = kt.l(r);
pr_ex.fx(t,r,s)   = pr_ex.l(t,r,s);
incadj.fx(t)      = incadj.l(t);
pres.fx(t,r,s)    = pres.l(t,r,s);
res_ex.fx(t,r,s)  = res_ex.l(t,r,s);
res.fx(t,r,s)     = res.l(t,r,s);
cpi.fx(t)         = cpi.l(t);
pr_ex_agg.fx(t,r) = pr_ex_agg.l(t,r);

* define the model for computing the uncompensated elasticity
model hh_opt /
  zp_c.c
  zp_cm.cm
  zp_cene.cene
  zp_cen.cen
  zp_cl.pcl

  hde_cm.pcm
  hde_cene.pcene
  hde_cen.pcen
  hde_c.pc
  hde_cd_en.cd
  hde_cd_m.cd
  hde_cd_ele.cd
  hde_leis.leis

  mc_time.l

  bc_hh.lambda

  foc_cl.cl
  foc_lambda.kh

  uce_c.uc_c
  uce_cen.uc_cen
  uce_cm.uc_cm
  uce_cl.uc_cl
  uce_cene.uc_cene

  aux_k_ex.k_ex
  aux_w.w
/;

* iteration limit
hh_opt.iterlim = 1000000;

* use model scaling
hh_opt.scaleopt = 1;

* turn off solution output
hh_opt.solprint = 0;



* ------------------------------------------------------------------------------
* calculate the uncompensated labor supply elasticity
* ------------------------------------------------------------------------------

parameters
  uncomp_effect(t,r,h)  uncompensated labor supply elasticity
  comp_effect(t,r,h)    compensated labor supply elasticity;

* shock the wage rate
pl.fx(t,r) = pl_base(t,r)*(1+.0001);

* solve the mcp problem
solve hh_opt using mcp;

* calculate the substitution effect
uncomp_effect(t,r,h)$(pl.l(t,r)-pl_base(t,r)) = (l.l(t,r,h)/l_base(t,r,h)-1)
                                                /(pl.l(t,r)/pl_base(t,r)-1);

* output csv results file
file lse_results /%lse_output_file%/;
lse_results.pc = 5;
lse_results.nd = 6;
put lse_results;
put "variable", "time", "region", "household", "value" /;
loop((t,r,h), put "uncomp", t.tl, r.tl, h.tl, uncomp_effect(t,r,h) / );
putclose;



* ------------------------------------------------------------------------------
* define mcp problem for the compensated elasticitiy
* ------------------------------------------------------------------------------

equations
  aux_inc_extra(t,r,h)  auxilary - compensated utility constraint;

* in this application we are just using the inc_extra variable as a mechanism to
* make the budget constraint non-binding
inc_extra.lo(t,r,h)$tfirst(t) = -inf;
inc_extra.up(t,r,h)$tfirst(t) = inf;

* constraint that welfare must equal baseline level
aux_inc_extra(t,r,h)$tfirst(t)..
  w(r,h) =e= w_base(r,h);

* define the model for computing the compensated elasticity
model hh_opt_comp /
  zp_c.c
  zp_cm.cm
  zp_cene.cene
  zp_cen.cen
  zp_cl.pcl

  hde_cm.pcm
  hde_cene.pcene
  hde_cen.pcen
  hde_c.pc
  hde_cd_en.cd
  hde_cd_m.cd
  hde_cd_ele.cd
  hde_leis.leis

  mc_time.l

  bc_hh.lambda

  foc_cl.cl
  foc_lambda.kh

  uce_c.uc_c
  uce_cen.uc_cen
  uce_cm.uc_cm
  uce_cl.uc_cl
  uce_cene.uc_cene

  aux_k_ex.k_ex
  aux_w.w
  aux_inc_extra.inc_extra
/;

* iteration limit
hh_opt_comp.iterlim = 1000000;

* use model scaling
hh_opt_comp.scaleopt = 1;

* turn off solution output
hh_opt_comp.solprint = 0;



* ------------------------------------------------------------------------------
* calculate the compensated labor supply elasticity
* ------------------------------------------------------------------------------

* solve the mcp problem
solve hh_opt_comp using mcp;

* calculate the income effect
comp_effect(t,r,h)$(pl.l(t,r)-pl_base(t,r)) = (l.l(t,r,h)/l_base(t,r,h)-1)
                                              /(pl.l(t,r)/pl_base(t,r)-1);

* output results file
lse_results.ap = 1;
put lse_results;
loop((t,r,h), put "comp", t.tl, r.tl, h.tl, comp_effect(t,r,h) / );
putclose;

* display results in listing file
display se_cl, le0, uncomp_effect, comp_effect;
