# sensitivity_analysis_example.R
#
# this example tests the sensitivity of social cost estimates for a hypothetical
# policy to the compensated labor supply elasticity parameter.
#
# during the initialization of a run the model:
#  1. loads the benchmark data from the gdx file defined in benchmark_file
#  2. loads the remaining parameters from the file defined in parameter_file
#  3. includes the gams file defined in prologue
#
# this allows sensitivity analysis to be performed by developing new benchmark
# or parameter files that are selected using the relevant command line
# variables. the other opetion is to use the prologue file adjust or change the
# the specific parameters over which sensitivity analysis is to be performed.
# this example uses the later approach to adjust a sinle parameter over which
# sensitivity analysis is performed.
#
# uses the sample regulation in examples/sample_abatement_requirement.gms
#
# this example is intended to be run from the top level directory of the model
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# options
# ------------------------------------------------------------------------------

# compensated labor supply elasticity values to consider in sensitivity analysis
lse_values = c(.1, .2, .3)



# ------------------------------------------------------------------------------
# run sensitivity analysis
# ------------------------------------------------------------------------------

# create a container for the results
ev = NULL

for (lse in lse_values) {

  print(paste0("running lse sensitivity: ",lse))

  # create a prologue file updating the compensated labor supply elasticity to
  # the sensitivity value
  prologue_file = "example_prologue.gms"
  fc = file(file.path("examples",prologue_file),open="wt")
  writeLines(paste0('lse_sub = ',lse,';'),con=fc)
  close(fc)

  # run the baseline
  run.model(output_file="baseline.csv",
            options=paste0(" --prologue=examples\\\\",prologue_file))

  # run the policy scenario
  run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
            policy_file="examples\\\\sample_abatement_requirement.gms",
            options=paste0(" --prologue=examples\\\\",prologue_file))

  # load the results file policy simulation
  policy = read.csv(file.path("output","policy.csv"),stringsAsFactors=FALSE)
  policy$Elasticity = lse

  # save the ev results for the current sensitivity analysis
  ev = rbind(ev,policy[policy$parameter=="ev_inf",])

}

# cleanup the prologue file
file.remove(file.path("examples","example_prologue.gms"))



# ------------------------------------------------------------------------------
# display the results
# ------------------------------------------------------------------------------

# aggregate ev estimates by households
ev = aggregate(ev$value,by=list(ev$household,ev$Elasticity),sum)
names(ev) = c("household","Elasticity","value")
ev$Elasticity = as.character(ev$Elasticity)

# extract initial year number of households by representative household
n = policy[policy$parameter=="n" & policy$time==min(na.omit(policy$time)),]
n = aggregate(n$value,by=list(n$household),sum)
names(n) = c("household","n")
n$n = n$n*1e6

# convert ev to per household terms
ev = merge(ev,n,all=TRUE)
ev$value = ev$value*1e9/ev$n

# set ordering for plot
ev$household = factor(ev$household,unique(ev$household))
ev$Elasticity = factor(ev$Elasticity,unique(ev$Elasticity))

# plot the change in labor demand across the two cases
p = ggplot(ev)+
      geom_bar(aes(x=household,y=value,fill=Elasticity),
               stat="identity",position="dodge")+
      labs(x="Household",y="Equivalent Variation [$/Household]")+
      scale_fill_manual(values=sage.palette)+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank())
print(p)
