# load.cps.asec
#
# returns a rectangular version of the cps asec data where the asscoiated 
# household and family information is attached to each person, obtained by using
# the nber sas import instructions to parse the fixed width ascii data file
#
# inputs:
#   year       year of the cps asec requested
#   data.dir   directory where the final data will be stored
#
# output:
#   the data frame returned is a rectangular version of the data where the 
#   asscoiated household and family information is attached to each person, 
#   while not the most memory efficient, this is a far easier form with which to
#   work with the data and the cps asec is realtively small
#
# dependincies:
#   SAScii
#   utilities/download_file
#
# notes:
#   - the cps asec collects information about economic information in the prior
#     year, so for information on 2016 the year requested in this function  
#     should be 2017
#

load.cps.asec= function(year,data.dir=".") {
  
  # file in which the rectangular cps asec data set will be stored
  data.file.name = paste0("cps_asec_",year,".RData")
  
  # check if the rectangular data set already exists and if so load the data set
  # and return it
  if (file.exists(file.path(data.dir,data.file.name))) {
    cat(paste0("loading ",year," cps asec data...\n"))
    load(file.path(data.dir,data.file.name))
    return(p.data)
  }
  
  # uses the sascii library to parse the nber sas input instructions to 
  # determine the column names and widths for the cps asec ascii data file
  library(SAScii)
  
  # library to download files
  source(file.path("utilities","download_file.R"))
  
  # download the cps asec ascii data file
  url = "https://data.nber.org/cps/"
  zip.file = paste0("cpsmar",year,".zip")
  file.name = paste0("asec",year,"_pubuse.dat")
  download_file(url,zip.file,data.dir,file.name)
  
  # download the nber sas import instructions for the ascii data file
  url = "https://data.nber.org/data/progs/cps/"
  sas.file = paste0("cpsmar",year,".sas")
  download_file(url,sas.file,data.dir)
  
  # read in the sas import instructions
  sas.instructions = readLines(file.path(data.dir,sas.file),encoding="UTF-8")
  
  # find the lines with the import instructions for the household records
  rectype.line = grep("if rectype = 1 then do",sas.instructions)
  h.begin.line = grep("INPUT",sas.instructions[-(1:rectype.line)])[1]+
                 rectype.line
  h.end.line = grep(";",sas.instructions[-(1:rectype.line)])[1]+rectype.line
  
  # find the lines with the import instructions for the family records
  f.begin.line = grep("INPUT",sas.instructions[-(1:h.end.line)])[1]+h.end.line
  f.end.line = grep(";",sas.instructions[-(1:f.begin.line)])[1]+f.begin.line
  
  # find the lines with the import instructions for the person records
  p.begin.line = grep("INPUT",sas.instructions[-(1:f.end.line)])[1]+f.end.line
  p.end.line = grep(";",sas.instructions[-(1:p.begin.line)])[1]+p.begin.line

  # get the line width from the sas import instructions
  lrecl = as.numeric(gsub(".*[=]([^;]+)[;].*","\\1",
                          sas.instructions[grep("LRECL",sas.instructions)]))
  
  # parse the sas input instructions to get the ascii data file column names and
  # widths for the household records
  h.parse = parse.SAScii(file.path(data.dir,sas.file),beginline=h.begin.line,
                         lrecl=lrecl)
  
  # parse the sas input instructions to get the ascii data file column names and
  # widths for the family records
  f.parse = parse.SAScii(file.path(data.dir,sas.file),beginline=f.begin.line,
                         lrecl=lrecl)
  
  # parse the sas input instructions to get the ascii data file column names and
  # widths for the person records
  p.parse = parse.SAScii(file.path(data.dir,sas.file),beginline=p.begin.line,
                         lrecl=lrecl)

  # create temporary files to store the household, family, and person records 
  # seperately
  h.temp.file = tempfile()
  f.temp.file = tempfile()
  p.temp.file = tempfile()
  
  # open the temporary files for writing
  h.fc = file(h.temp.file,"w")
  f.fc = file(f.temp.file,"w")
  p.fc = file(p.temp.file,"w")
  
  # open the cps asec data file for reading
  fc = file(file.path(data.dir,file.name),"r")
  
  cat(paste0("seperating ",year," cps asec records\n"))
  cat(" ")
  cat("progress... 0 lines processed","\r")
  
  # line counter for records processed
  count = 0
  
  # loop through each line in the data set and write it to the temporary file
  # for the associated type of records: household, family, person
  while (length(line<-readLines(fc,1))>0) {
    
    # get the type of record
    rectype = substr(line,1,1)
    
    # save the current record in the appropriate temporary file
    if (rectype==1) {
      writeLines(substr(line,1,lrecl),con=h.fc)
    } else if (rectype==2) {
      writeLines(substr(line,1,lrecl),con=f.fc)
    } else if (rectype==3) {
      writeLines(substr(line,1,lrecl),con=p.fc)
    }
    
    count = count+1
    
    cat("progress... ",prettyNum(count,big.mark=",")," lines processed","\r")
    
  }
  
  cat("\n")
  
  # close the file connections
  close(fc)
  close(h.fc)
  close(f.fc)
  close(p.fc)
  
  # remove place holders from the parsed names
  h.names = h.parse$varname[!is.na(h.parse$varname)]
  f.names = f.parse$varname[!is.na(f.parse$varname)]
  p.names = p.parse$varname[!is.na(p.parse$varname)]
  
  # read the fixed width format household records
  cat("loading the household records...\n")
  h.data = read.fwf(h.temp.file,h.parse$width,col.names=h.names,comment.char='',
                    colClasses='character',buffersize=1000)
  
  # read the fixed width format family records
  cat("loading the family records...\n")
  f.data = read.fwf(f.temp.file,f.parse$width,col.names=f.names,comment.char='',
                    colClasses='character',buffersize=1000)
  
  # read the fixed width format person records
  cat("loading the person records...\n")
  p.data = read.fwf(p.temp.file,p.parse$width,col.names=p.names,comment.char='',
                    colClasses='character',buffersize=1000)
  
  # convert numeric household columns and apply any divisors
  # typically the only divisor is for the weights which are multiplied by 100 in 
  # the ascii data file
  h.parse = h.parse[!is.na(h.parse$varname),]
  for (name in names(h.data)) {
    if (!h.parse$char[h.parse$varname==name])
      h.data[,name] = as.numeric(h.data[,name])
    if (h.parse$divisor[h.parse$varname==name]!=1)
      h.data[,name] = h.data[,name]*h.parse$divisor[h.parse$varname==name]
  }
  
  # convert numeric family columns and apply any divisors
  # typically the only divisor is for the weights which are multiplied by 100 in 
  # the ascii data file
  f.parse = f.parse[!is.na(f.parse$varname),]
  for (name in names(f.data)) {
    if (!f.parse$char[f.parse$varname==name])
      f.data[,name] = as.numeric(f.data[,name])
    if (f.parse$divisor[f.parse$varname==name]!=1)
      f.data[,name] = f.data[,name]*f.parse$divisor[f.parse$varname==name]
  }
  
  # convert numeric person columns and apply any divisors
  # typically the only divisor is for the weights which are multiplied by 100 in 
  # the ascii data file
  p.parse = p.parse[!is.na(p.parse$varname),]
  for (name in names(p.data)) {
    if (!p.parse$char[p.parse$varname==name])
      p.data[,name] = as.numeric(p.data[,name])
    if (p.parse$divisor[p.parse$varname==name]!=1)
      p.data[,name] = p.data[,name]*p.parse$divisor[p.parse$varname==name]
  }
  
  # rename the household sequence number to match the household record variable
  # and merge the household and person records
  p.data$H_SEQ = p.data$PH_SEQ
  p.data = merge(h.data,p.data,sort=FALSE)
  
  # rename the household sequence number and family sequence number to match the
  # household and person record variable and merge in the family records
  f.data$H_SEQ = f.data$FH_SEQ
  f.data$PF_SEQ = f.data$FFPOS
  p.data = merge(f.data,p.data,sort=FALSE)
  
  # convert all names to lower case
  names(p.data) = tolower(names(p.data))
  
  # save the processed rectangular data
  save(p.data,file=file.path(data.dir,data.file.name))
  
  # remove the temporary data files
  file.remove(h.temp.file)
  file.remove(f.temp.file)
  file.remove(p.temp.file)
  file.remove(file.path(data.dir,file.name))
  file.remove(file.path(data.dir,sas.file))
  
  return(p.data)
  
}
